@@ -3,45 +3,45 @@ mod iter;
 mod utils;
 
 use crate::{
-    document::{DocumentResult, OffDocument, ParserOptions},
     geometry::{Color, Face, Position, Vertex},
+    mesh::{Mesh, ParserOptions},
 };
 
 use self::{
-    error::{ParserError, ParserErrorKind},
+    error::{Error, Kind},
     iter::OffLines,
     utils::{ConvertVec, StrParts},
 };
 
-pub type ParserResult<T = ()> = Result<T, ParserError>;
+pub type Result<T = ()> = std::result::Result<T, Error>;
 
 #[derive(Debug, Clone)]
-pub struct DocumentParser<'a> {
+pub struct Parser<'a> {
     lines: OffLines<'a>,
     prev_line_index: usize,
     vertex_count: usize,
     face_count: usize,
     edge_count: usize,
-    document: OffDocument,
+    document: Mesh,
     options: ParserOptions,
 }
 
-impl<'a> DocumentParser<'a> {
+impl<'a> Parser<'a> {
     pub fn new<S: AsRef<str>>(s: &'a S, options: ParserOptions) -> Self {
         let lines = OffLines::new(s.as_ref());
 
-        DocumentParser {
+        Parser {
             lines,
             prev_line_index: 0,
             vertex_count: 0,
             face_count: 0,
             edge_count: 0,
-            document: OffDocument::new(),
+            document: Mesh::new(),
             options,
         }
     }
 
-    pub fn parse(mut self) -> DocumentResult {
+    pub fn parse(mut self) -> crate::mesh::Result {
         self.parse_header()?;
         self.parse_counts()?;
         self.parse_vertices()?;
@@ -49,7 +49,7 @@ impl<'a> DocumentParser<'a> {
 
         // TODO: valitdate the counts
 
-        self.finalize()
+        Ok(self.finalize())
     }
 
     fn next_line(&mut self) -> Option<(usize, &'a str)> {
@@ -60,14 +60,14 @@ impl<'a> DocumentParser<'a> {
         Some((line_index, line))
     }
 
-    fn parse_header(&mut self) -> ParserResult {
+    fn parse_header(&mut self) -> Result {
         let (line_index, line) = self
             .next_line()
-            .ok_or_else(|| ParserError::without_message(ParserErrorKind::Empty, 0))?;
+            .ok_or_else(|| Error::without_message(Kind::Empty, 0))?;
 
         if line != "OFF" {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidHeader,
+            return Err(Error::with_message(
+                Kind::InvalidHeader,
                 line_index,
                 "First non-comment line should be `OFF`",
             ));
@@ -76,13 +76,9 @@ impl<'a> DocumentParser<'a> {
         Ok(())
     }
 
-    fn parse_counts(&mut self) -> ParserResult {
+    fn parse_counts(&mut self) -> Result {
         let (line_index, line) = self.next_line().ok_or_else(|| {
-            ParserError::with_message(
-                ParserErrorKind::Missing,
-                self.prev_line_index + 1,
-                "No counts present",
-            )
+            Error::with_message(Kind::Missing, self.prev_line_index + 1, "No counts present")
         })?;
 
         let counts: Vec<&str> = line.split_line();
@@ -101,8 +97,8 @@ impl<'a> DocumentParser<'a> {
         //     .collect::<Result<Vec<usize>, ParserError>>()?;
 
         let num: Vec<usize> = counts.convert_vec().map_err(|err| {
-            ParserError::with_message(
-                ParserErrorKind::InvalidCounts,
+            Error::with_message(
+                Kind::InvalidCounts,
                 line_index,
                 format!("Failed to parse count as number ({})", err),
             )
@@ -119,8 +115,8 @@ impl<'a> DocumentParser<'a> {
                 self.face_count = face_count;
             }
             _ => {
-                return Err(ParserError::with_message(
-                    ParserErrorKind::InvalidCounts,
+                return Err(Error::with_message(
+                    Kind::InvalidCounts,
                     line_index,
                     format!(
                         "Invalid amount of counts present (expected: 2-3, actual: {})",
@@ -132,8 +128,8 @@ impl<'a> DocumentParser<'a> {
 
         // Check for limits
         if self.vertex_count > self.options.limits.vertex_count {
-            return Err(ParserError::with_message(
-                ParserErrorKind::LimitExceeded,
+            return Err(Error::with_message(
+                Kind::LimitExceeded,
                 line_index,
                 format!(
                     "Vertext count exceeds limit (limit: {}, actual: {})",
@@ -143,8 +139,8 @@ impl<'a> DocumentParser<'a> {
         }
 
         if self.face_count > self.options.limits.face_count {
-            return Err(ParserError::with_message(
-                ParserErrorKind::LimitExceeded,
+            return Err(Error::with_message(
+                Kind::LimitExceeded,
                 line_index,
                 format!(
                     "Face count exceeds limit (limit: {}, actual: {})",
@@ -156,33 +152,28 @@ impl<'a> DocumentParser<'a> {
         Ok(())
     }
 
-    fn parse_vertices(&mut self) -> ParserResult {
+    fn parse_vertices(&mut self) -> Result {
         for _ in 0..self.vertex_count {
             let (line_index, line) = self.next_line().ok_or_else(|| {
-                ParserError::with_message(
-                    ParserErrorKind::Missing,
+                Error::with_message(
+                    Kind::Missing,
                     self.prev_line_index + 1,
                     "Expected vertex definition",
                 )
             })?;
 
             let parts = line.split_line();
-            let vertex = self.parse_vertex(line_index, parts)?;
+            let vertex = self.parse_vertex(line_index, &parts)?;
             self.document.vertices.push(vertex);
         }
 
         Ok(())
     }
 
-    fn parse_vertex(&mut self, line_index: usize, parts: Vec<&str>) -> ParserResult<Vertex> {
-        // TODO: dont work if we have colors
-        // if vertex_str.len() != 3 {
-        //     return Err(ParserError::InvalidVertex);
-        // }
-
+    fn parse_vertex(&mut self, line_index: usize, parts: &[&str]) -> Result<Vertex> {
         if parts.len() < 3 {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidVertexPosition,
+            return Err(Error::with_message(
+                Kind::InvalidVertexPosition,
                 line_index,
                 format!(
                     "Not enough parts for position (expected: >= 3, actual: {})",
@@ -191,7 +182,7 @@ impl<'a> DocumentParser<'a> {
             ));
         }
 
-        let position = self.parse_position(line_index, &parts[0..=2])?;
+        let position = Parser::parse_position(line_index, &parts[0..=2])?;
 
         let color = if parts.len() > 3 {
             Some(self.parse_color(line_index, &parts[3..])?)
@@ -202,10 +193,10 @@ impl<'a> DocumentParser<'a> {
         Ok(Vertex { position, color })
     }
 
-    fn parse_position(&mut self, line_index: usize, parts: &[&str]) -> ParserResult<Position> {
+    fn parse_position(line_index: usize, parts: &[&str]) -> Result<Position> {
         if parts.len() != 3 {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidVertexPosition,
+            return Err(Error::with_message(
+                Kind::InvalidVertexPosition,
                 line_index,
                 format!(
                     "Invalid number of coordinates given (expected: 3, actual: {})",
@@ -218,23 +209,22 @@ impl<'a> DocumentParser<'a> {
             .iter()
             .map(|s| {
                 s.parse().map_err(|err| {
-                    ParserError::with_message(
-                        ParserErrorKind::InvalidVertexPosition,
+                    Error::with_message(
+                        Kind::InvalidVertexPosition,
                         line_index,
                         format!("Failed to parse coordinate as number: ({})", err),
                     )
                 })
             })
-            .collect::<Result<Vec<f32>, ParserError>>()?;
+            .collect::<Result<Vec<f32>>>()?;
 
-        // TODO: should not be try_from as we check for validity above?
         Position::try_from(position_parts)
     }
 
-    fn parse_color(&mut self, line_index: usize, parts: &[&str]) -> ParserResult<Color> {
+    fn parse_color(&mut self, line_index: usize, parts: &[&str]) -> Result<Color> {
         if parts.len() != self.options.color_format.element_count() {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidColor,
+            return Err(Error::with_message(
+                Kind::InvalidColor,
                 line_index,
                 format!(
                     "Invalid number of color elements given (expected: {}, actual: {})",
@@ -250,44 +240,44 @@ impl<'a> DocumentParser<'a> {
                 .iter()
                 .map(|s| {
                     s.parse::<f32>().map_err(|err| {
-                        ParserError::with_message(
-                            ParserErrorKind::InvalidColor,
+                        Error::with_message(
+                            Kind::InvalidColor,
                             line_index,
                             format!("Failed to parse color as float: {}", err),
                         )
                     })
                 })
-                .collect::<Result<Vec<f32>, ParserError>>()
+                .collect::<Result<Vec<f32>>>()
         } else {
             // parse as u8 and convert to f32
             parts
                 .iter()
                 .map(|s| {
-                    s.parse::<u8>().map(|val| val as f32).map_err(|err| {
-                        ParserError::with_message(
-                            ParserErrorKind::InvalidColor,
+                    s.parse::<u8>().map(f32::from).map_err(|err| {
+                        Error::with_message(
+                            Kind::InvalidColor,
                             line_index,
                             format!("Failed to parse color as u8: {}", err),
                         )
                     })
                 })
-                .collect::<Result<Vec<f32>, ParserError>>()
+                .collect::<Result<Vec<f32>>>()
         }?;
 
         Color::try_from(color_float).map_err(|err| {
-            ParserError::with_message(
-                ParserErrorKind::InvalidColor,
+            Error::with_message(
+                Kind::InvalidColor,
                 line_index,
                 format!("Failed to parse color: {}", err),
             )
         })
     }
 
-    fn parse_faces(&mut self) -> ParserResult {
+    fn parse_faces(&mut self) -> Result {
         for _ in 0..self.face_count {
             let (line_index, line) = self.next_line().ok_or_else(|| {
-                ParserError::with_message(
-                    ParserErrorKind::Missing,
+                Error::with_message(
+                    Kind::Missing,
                     self.prev_line_index + 1,
                     "Expected face definition",
                 )
@@ -301,26 +291,26 @@ impl<'a> DocumentParser<'a> {
         Ok(())
     }
 
-    fn parse_face(&mut self, line_index: usize, mut parts: &[&str]) -> ParserResult<Face> {
+    fn parse_face(&mut self, line_index: usize, mut parts: &[&str]) -> Result<Face> {
         if parts.len() < 4 {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidFace,
+            return Err(Error::with_message(
+                Kind::InvalidFace,
                 line_index,
                 format!("Not enough arguments. At least three vertex indicies required (e.g. `3 1 2 3`). {} arguments given", parts.len()),
             ));
         }
 
         let vertex_count: usize = parts[0].parse().map_err(|err| {
-            ParserError::with_message(
-                ParserErrorKind::InvalidFace,
+            Error::with_message(
+                Kind::InvalidFace,
                 line_index,
                 format!("Failed to parse vertex count for face definition: {}", err),
             )
         })?;
 
         if vertex_count > self.options.limits.face_vertex_count {
-            return Err(ParserError::with_message(
-                ParserErrorKind::LimitExceeded,
+            return Err(Error::with_message(
+                Kind::LimitExceeded,
                 line_index,
                 format!(
                     "Vertex count of face exceeds limit (limit: {}, actual: {})",
@@ -332,57 +322,53 @@ impl<'a> DocumentParser<'a> {
         // "Consume" vertex_count
         parts = &parts[1..];
 
+        // TODO
         // // sanity check
         // if face_str.len() != vertex_count as usize {
         //     return Err(ParserError::InvalidFace);
         // }
 
+        // TODO
         // faces are polygons and might have to be triangulated later. Therefore we require at least three vertices
         // if vertex_count < 3 {
         //     return Err(ParserError::InvalidFace);
         // }
-        let vertices = self.parse_face_index(line_index, vertex_count, parts)?;
+        let vertices = Parser::parse_face_indices(line_index, vertex_count, parts)?;
 
         // "Consume" vertex indexes
         parts = &parts[vertex_count..];
 
-        let color = if !parts.is_empty() {
-            Some(self.parse_color(line_index, parts)?)
-        } else {
+        let color = if parts.is_empty() {
             None
+        } else {
+            Some(self.parse_color(line_index, parts)?)
         };
 
         Ok(Face { vertices, color })
     }
 
-    fn parse_face_index(
-        &mut self,
+    fn parse_face_indices(
         line_index: usize,
         vertex_count: usize,
         parts: &[&str],
-    ) -> ParserResult<Vec<usize>> {
-        // TODO: dont work if we have colors
-        // if vertex_str.len() != 3 {
-        //     return Err(ParserError::InvalidVertex);
-        // }
-
+    ) -> Result<Vec<usize>> {
         let vertices: Vec<usize> = parts
             .iter()
             .take(vertex_count)
             .map(|s| {
                 s.parse().map_err(|err| {
-                    ParserError::with_message(
-                        ParserErrorKind::InvalidFace,
+                    Error::with_message(
+                        Kind::InvalidFaceIndex,
                         line_index,
                         format!("Failed to parse vertex index as number: ({})", err),
                     )
                 })
             })
-            .collect::<Result<Vec<usize>, ParserError>>()?;
+            .collect::<Result<Vec<usize>>>()?;
 
         if vertices.len() != vertex_count {
-            return Err(ParserError::with_message(
-                ParserErrorKind::InvalidFaceIndex,
+            return Err(Error::with_message(
+                Kind::InvalidFaceIndex,
                 line_index,
                 format!(
                     "Invalid number of face indexes given (expected: {}, actual: {})",
@@ -395,18 +381,18 @@ impl<'a> DocumentParser<'a> {
         Ok(vertices)
     }
 
-    fn finalize(self) -> DocumentResult {
-        Ok(self.document)
+    fn finalize(self) -> Mesh {
+        self.document
     }
 }
 
 impl TryFrom<Vec<f32>> for Color {
-    type Error = ParserError;
+    type Error = Error;
 
-    fn try_from(value: Vec<f32>) -> Result<Self, Self::Error> {
+    fn try_from(value: Vec<f32>) -> std::result::Result<Self, Self::Error> {
         if 3 > value.len() || 4 < value.len() {
             return Err(Self::Error::with_message(
-                ParserErrorKind::InvalidColor,
+                Kind::InvalidColor,
                 0,
                 format!(
                     "Invalid amount of arguments (expected: 3-4, actual: {})",
@@ -427,12 +413,12 @@ impl TryFrom<Vec<f32>> for Color {
 }
 
 impl TryFrom<Vec<f32>> for Position {
-    type Error = ParserError;
+    type Error = Error;
 
-    fn try_from(value: Vec<f32>) -> Result<Self, Self::Error> {
+    fn try_from(value: Vec<f32>) -> std::result::Result<Self, Self::Error> {
         if value.len() != 3 {
             return Err(Self::Error::with_message(
-                ParserErrorKind::InvalidVertexPosition,
+                Kind::InvalidVertexPosition,
                 0,
                 format!(
                     "Invalid amount of arguments (expected: 3, actual: {})",
@@ -455,61 +441,211 @@ mod tests {
 
     use super::*;
 
-    // #[test]
-    // #[should_panic]
-    // #[allow(unused)]
-    // fn test_state() {
-    //     let mut parser = DocumentParser {
-    //         lines: vec!["OFF"],
-    //         line: 0,
-    //         state: ParserState::Header,
-    //         document: OffDocument::new(),
-    //     };
-    //     parser.parse_counts();
-    // }
+    // TODO: test parse_header
+    // TODO: test parse_counts
+    // TODO: test parse_vertices
+    // TODO: test parse_vertex
+    // TODO: test DocumentParser::parse_position
+    // TODO: test parse_color
+    // TODO: test parse_Faces
+    // TODO: test parse_face
 
-    // #[test]
-    // fn parse_header() {
-    //     let mut parser = DocumentParser {
-    //         lines: vec!["ignore", "me", "OFF", "!"],
-    //         line: 2,
-    //         state: ParserState::Header,
-    //         document: OffDocument::new(),
-    //     };
-    //     assert!(matches!(parser.parse_header(), Ok(_)));
-    //     let mut parser = DocumentParser {
-    //         lines: vec!["OOFF"],
-    //         line: 0,
-    //         state: ParserState::Header,
-    //         document: OffDocument::new(),
-    //     };
-    //     assert!(matches!(
-    //         parser.parse_header(),
-    //         Err(ParserError::InvalidHeader)
-    //     ));
-    // }
+    #[test]
+    fn parse_face() {
+        let mut parser = Parser::new(&"", ParserOptions::default());
+        let result = parser.parse_face(0, &["3", "1", "2", "3"]);
+        assert!(result.is_ok());
+        assert_eq!(
+            result.unwrap(),
+            Face {
+                vertices: vec![1, 2, 3],
+                color: None
+            }
+        );
+    }
 
-    // #[test]
-    // fn parse_counts() {
-    //     let mut parser = DocumentParser {
-    //         lines: vec!["a12 3 4"],
-    //         line: 0,
-    //         state: ParserState::Counts,
-    //         document: OffDocument::new(),
-    //     };
-    //     assert!(matches!(
-    //         parser.parse_counts(),
-    //         Err(ParserError::InvalidCounts)
-    //     ));
-    //     let mut parser = DocumentParser {
-    //         lines: vec!["1 1337 42"],
-    //         line: 0,
-    //         state: ParserState::Counts,
-    //         document: OffDocument::new(),
-    //     };
-    //     assert!(matches!(parser.parse_counts(), Ok(_)));
-    //     assert_eq!(parser.document.vertex_count(), 1);
-    //     assert_eq!(parser.document.face_count(), 1337);
-    //     assert_eq!(parser.document.edge_count(), 42);
-    // }
+    #[test]
+    fn parse_face_more() {
+        let mut parser = Parser::new(&"", ParserOptions::default());
+        let result = parser.parse_face(0, &["4", "2", "3", "1", "1337"]);
+        assert!(result.is_ok());
+        assert_eq!(
+            result.unwrap(),
+            Face {
+                vertices: vec![2, 3, 1, 1337],
+                color: None
+            }
+        );
+    }
+
+    #[test]
+    fn parse_face_too_little_parts() {
+        let mut parser = Parser::new(&"", ParserOptions::default());
+        let result = parser.parse_face(0, &["6", "1", "2", "3"]);
+        assert!(result.is_err());
+        assert!(matches!(
+            result.unwrap_err(),
+            Error {
+                kind: Kind::InvalidFaceIndex,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn parse_face_too_many_parts() {
+        let mut parser = Parser::new(&"", ParserOptions::default());
+        let result = parser.parse_face(0, &["3", "2", "3", "2", "3"]);
+        assert!(result.is_err());
+        assert!(matches!(
+            result.unwrap_err(),
+            Error {
+                kind: Kind::InvalidColor,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn parse_face_no_number() {
+        let mut parser = Parser::new(&"", ParserOptions::default());
+        let result = parser.parse_face(0, &["3", "1", "asdf", "3"]);
+        assert!(result.is_err());
+        assert!(matches!(
+            result.unwrap_err(),
+            Error {
+                kind: Kind::InvalidFaceIndex,
+                ..
+            }
+        ));
+    }
+
+    // TODO: face colors
+
+    #[test]
+    fn parse_face_index() {
+        let result = Parser::parse_face_indices(0, 3, &["1", "2", "3"]);
+        assert!(result.is_ok());
+        assert_eq!(result.unwrap(), vec![1, 2, 3]);
+    }
+
+    #[test]
+    fn parse_face_index_more() {
+        let result = Parser::parse_face_indices(0, 5, &["1", "2", "3", "1", "1337"]);
+        assert!(result.is_ok());
+        assert_eq!(result.unwrap(), vec![1, 2, 3, 1, 1337]);
+    }
+
+    #[test]
+    fn parse_face_index_too_little_parts() {
+        let result = Parser::parse_face_indices(0, 5, &["1", "2", "3"]);
+        assert!(result.is_err());
+        assert!(matches!(
+            result.unwrap_err(),
+            Error {
+                kind: Kind::InvalidFaceIndex,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn parse_face_index_too_many_parts() {
+        let result = Parser::parse_face_indices(0, 3, &["1", "2", "3", "2", "3"]);
+        assert!(result.is_ok());
+        assert_eq!(result.unwrap(), vec![1, 2, 3]);
+    }
+
+    #[test]
+    fn parse_face_index_no_number() {
+        let result = Parser::parse_face_indices(0, 3, &["1", "asdf", "3"]);
+        assert!(result.is_err());
+        assert!(matches!(
+            result.unwrap_err(),
+            Error {
+                kind: Kind::InvalidFaceIndex,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn try_from_color_rgb() {
+        let vec = vec![1.0, 2.0, 3.0];
+        let color = Color::try_from(vec);
+        assert!(color.is_ok());
+        assert_eq!(color.unwrap(), Color::new(1.0, 2.0, 3.0, 1.0));
+    }
+
+    #[test]
+    fn try_from_color_rgba() {
+        let vec = vec![1.0, 2.0, 3.0, 4.0];
+        let color = Color::try_from(vec);
+        assert!(color.is_ok());
+        assert_eq!(color.unwrap(), Color::new(1.0, 2.0, 3.0, 4.0));
+    }
+
+    #[test]
+    fn try_from_color_err_too_little_arguments() {
+        let vec = vec![1.0, 2.0];
+        let color = Color::try_from(vec);
+        assert!(color.is_err());
+        assert!(matches!(
+            color.unwrap_err(),
+            Error {
+                kind: Kind::InvalidColor,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn try_from_color_err_too_many_arguments() {
+        let vec = vec![1.0, 2.0, 3.0, 4.0, 5.0];
+        let color = Color::try_from(vec);
+        assert!(color.is_err());
+        assert!(matches!(
+            color.unwrap_err(),
+            Error {
+                kind: Kind::InvalidColor,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn try_from_positiom() {
+        let vec = vec![1.0, 2.0, 3.0];
+        let position = Position::try_from(vec);
+        assert!(position.is_ok());
+        assert_eq!(position.unwrap(), Position::new(1.0, 2.0, 3.0));
+    }
+
+    #[test]
+    fn try_from_positiom_too_little_arguments() {
+        let vec = vec![1.0, 2.0];
+        let position = Position::try_from(vec);
+        assert!(position.is_err());
+        assert!(matches!(
+            position.unwrap_err(),
+            Error {
+                kind: Kind::InvalidVertexPosition,
+                ..
+            }
+        ));
+    }
+
+    #[test]
+    fn try_from_positiom_too_many_arguments() {
+        let vec = vec![1.0, 2.0, 3.0, 4.0];
+        let position = Position::try_from(vec);
+        assert!(position.is_err());
+        assert!(matches!(
+            position.unwrap_err(),
+            Error {
+                kind: Kind::InvalidVertexPosition,
+                ..
+            }
+        ));
+    }
 }
